@@ -15,12 +15,13 @@ use crate::{
 };
 
 /// `ConnectPacket` consists of three parts:
-/// * FixedHeader
-/// * VariableHeader
-/// * Payload
+/// * `FixedHeader`
+/// * `VariableHeader`
+/// * `Payload`
+///
 /// Note that fixed header part is same in all packets so that we just ignore it.
 ///
-/// Basic struct of ConnectPacket is as below:
+/// Basic struct of `ConnectPacket` is as below:
 /// ```txt
 ///  7                          0
 /// +----------------------------+
@@ -76,6 +77,7 @@ use crate::{
 /// whose presence is determined by the flags in the Variable Header. These fields,
 /// if present, MUST appear in the order Client Identifier, Will Properties, Will Topic,
 /// Will Payload, User Name, Password [MQTT-3.1.3-1].
+#[allow(clippy::module_name_repetitions)]
 #[derive(Clone, Debug, Default, PartialEq)]
 pub struct ConnectPacket {
     /// Protocol name can only be `MQTT` in specification.
@@ -177,6 +179,7 @@ pub struct ConnectPacket {
     password: BinaryData,
 }
 
+/// Properties available in connect packet.
 pub const CONNECT_PROPERTIES: &[PropertyType] = &[
     PropertyType::SessionExpiryInterval,
     PropertyType::ReceiveMaximum,
@@ -188,6 +191,7 @@ pub const CONNECT_PROPERTIES: &[PropertyType] = &[
     PropertyType::AuthenticationData,
 ];
 
+/// Properties available in connect-will.
 pub const CONNECT_WILL_PROPERTIES: &[PropertyType] = &[
     PropertyType::WillDelayInterval,
     PropertyType::PayloadFormatIndicator,
@@ -199,160 +203,220 @@ pub const CONNECT_WILL_PROPERTIES: &[PropertyType] = &[
 ];
 
 impl ConnectPacket {
-    pub fn new(client_id: &str) -> Result<ConnectPacket, EncodeError> {
-        let protocol_name = StringData::from_str(PROTOCOL_NAME)?;
+    /// Create a new connect packet.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `client_id` is invalid.
+    pub fn new(client_id: &str) -> Result<Self, EncodeError> {
+        let protocol_name = StringData::from(PROTOCOL_NAME)?;
         validate_client_id(client_id).map_err(|_err| EncodeError::InvalidClientId)?;
-        let client_id = StringData::from_str(client_id)?;
-        Ok(ConnectPacket {
+        let client_id = StringData::from(client_id)?;
+        Ok(Self {
             protocol_name,
             keep_alive: U16Data::new(60),
             client_id,
-            ..ConnectPacket::default()
+            ..Self::default()
         })
     }
 
+    /// Update protocol level.
     pub fn set_protcol_level(&mut self, level: ProtocolLevel) -> &mut Self {
         self.protocol_level = level;
         self
     }
 
-    pub fn protocol_level(&self) -> ProtocolLevel {
+    /// Get current mqtt protocol level.
+    #[must_use]
+    pub const fn protocol_level(&self) -> ProtocolLevel {
         self.protocol_level
     }
 
+    /// Update keep-alive value.
     pub fn set_keep_alive(&mut self, keep_alive: u16) -> &mut Self {
         self.keep_alive = U16Data::new(keep_alive);
         self
     }
 
-    pub fn keep_alive(&self) -> u16 {
+    /// Get current keep-alive value.
+    #[must_use]
+    pub const fn keep_alive(&self) -> u16 {
         self.keep_alive.value()
     }
 
+    /// Update will-retain flag.
     pub fn set_will_retain(&mut self, will_retain: bool) -> &mut Self {
         self.connect_flags.set_will_retain(will_retain);
         self
     }
 
-    pub fn will_retain(&self) -> bool {
+    /// Get current will-retain flag.
+    #[must_use]
+    pub const fn will_retain(&self) -> bool {
         self.connect_flags.will_retain()
     }
 
+    /// Update will-qos value.
     pub fn set_will_qos(&mut self, qos: QoS) -> &mut Self {
         self.connect_flags.set_will_qos(qos);
         self
     }
 
-    pub fn will_qos(&self) -> QoS {
+    /// Get current will-qos value.
+    #[must_use]
+    pub const fn will_qos(&self) -> QoS {
         self.connect_flags.will_qos()
     }
 
+    /// Update will flag.
     pub fn set_will(&mut self, will: bool) -> &mut Self {
         self.connect_flags.set_will(will);
         self
     }
 
-    pub fn will(&self) -> bool {
+    /// Get current will flag.
+    #[must_use]
+    pub const fn will(&self) -> bool {
         self.connect_flags.will()
     }
 
+    /// Update clean-session flag.
     pub fn set_clean_session(&mut self, clean_session: bool) -> &mut Self {
         self.connect_flags.set_clean_session(clean_session);
         self
     }
 
-    pub fn clean_session(&self) -> bool {
+    /// Get clean-session flag.
+    #[must_use]
+    pub const fn clean_session(&self) -> bool {
         self.connect_flags.clean_session()
     }
 
+    /// Get a mutable reference to property list.
     pub fn properties_mut(&mut self) -> &mut Properties {
         &mut self.properties
     }
 
-    pub fn properties(&self) -> &Properties {
+    /// Get a reference to property list.
+    #[must_use]
+    pub const fn properties(&self) -> &Properties {
         &self.properties
     }
 
-    pub fn set_client_id(&mut self, id: &str) -> Result<&mut Self, EncodeError> {
-        validate_client_id(id).map_err(|_err| EncodeError::InvalidClientId)?;
-        self.client_id = StringData::from_str(id)?;
+    /// Update client id.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `client_id` is invalid.
+    pub fn set_client_id(&mut self, client_id: &str) -> Result<&mut Self, EncodeError> {
+        validate_client_id(client_id).map_err(|_err| EncodeError::InvalidClientId)?;
+        self.client_id = StringData::from(client_id)?;
         Ok(self)
     }
 
+    /// Get current client id.
+    #[must_use]
     pub fn client_id(&self) -> &str {
         self.client_id.as_ref()
     }
 
+    /// Update username value.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `username` is out of range.
     pub fn set_username(&mut self, username: Option<&str>) -> Result<&mut Self, DecodeError> {
-        match username {
-            Some(username) => {
-                self.username = StringData::from_str(username)?;
-                self.connect_flags.set_username(true);
-            }
-            _ => {
-                self.connect_flags.set_username(false);
-                self.username = StringData::new();
-            }
+        if let Some(username) = username {
+            self.username = StringData::from(username)?;
+            self.connect_flags.set_has_username(true);
+        } else {
+            self.connect_flags.set_has_username(false);
+            self.username = StringData::new();
         }
         Ok(self)
     }
 
+    /// Get current username value.
+    #[must_use]
     pub fn username(&self) -> Option<&str> {
-        if self.connect_flags.username() {
+        if self.connect_flags.has_username() {
             Some(self.username.as_ref())
         } else {
             None
         }
     }
 
+    /// Update password value.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `password` is out of range.
     pub fn set_password(&mut self, password: Option<&[u8]>) -> Result<&mut Self, EncodeError> {
         match password {
             Some(password) => {
-                self.connect_flags.set_password(true);
+                self.connect_flags.set_has_password(true);
                 self.password = BinaryData::from_slice(password)?;
             }
             None => {
-                self.connect_flags.set_password(false);
+                self.connect_flags.set_has_password(false);
                 self.password.clear();
             }
         }
         Ok(self)
     }
 
+    /// Get current password value.
+    #[must_use]
     pub fn password(&self) -> Option<&[u8]> {
-        if self.connect_flags.password() {
+        if self.connect_flags.has_password() {
             Some(self.password.as_ref())
         } else {
             None
         }
     }
 
+    /// Get a mutable reference to will property list.
     pub fn will_properties_mut(&mut self) -> &mut Properties {
         &mut self.will_properties
     }
 
-    pub fn will_properties(&self) -> &Properties {
+    /// Get a reference to will property list.
+    #[must_use]
+    pub const fn will_properties(&self) -> &Properties {
         &self.will_properties
     }
 
+    /// Update will topic.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `topic` is invalid.
     pub fn set_will_topic(&mut self, topic: &str) -> Result<&mut Self, EncodeError> {
-        if !topic.is_empty() {
-            self.will_topic = Some(PubTopic::new(topic)?);
-        } else {
+        if topic.is_empty() {
             self.will_topic = None;
+        } else {
+            self.will_topic = Some(PubTopic::new(topic)?);
         }
         Ok(self)
     }
 
+    /// Get current will topic.
     pub fn will_topic(&self) -> Option<&str> {
         self.will_topic.as_ref().map(AsRef::as_ref)
     }
 
+    /// Update will message bytes.
+    ///
+    /// # Errors
+    ///
+    /// Returns error if `message` is out of range.
     pub fn set_will_message(&mut self, message: &[u8]) -> Result<&mut Self, EncodeError> {
         self.will_message = BinaryData::from_slice(message)?;
         Ok(self)
     }
 
+    /// Get will message bytes.
+    #[must_use]
     pub fn will_message(&self) -> &[u8] {
         self.will_message.as_ref()
     }
@@ -369,9 +433,9 @@ impl EncodePacket for ConnectPacket {
         let old_len = v.len();
 
         let mut remaining_length = self.protocol_name.bytes()
-            + self.protocol_level.bytes()
-            + self.connect_flags.bytes()
-            + self.keep_alive.bytes()
+            + ProtocolLevel::bytes()
+            + ConnectFlags::bytes()
+            + U16Data::bytes()  // keep_alive
             + self.client_id.bytes();
 
         // Check username/password/topic/message.
@@ -382,10 +446,10 @@ impl EncodePacket for ConnectPacket {
             }
             remaining_length += self.will_message.bytes();
         }
-        if self.connect_flags.username() {
+        if self.connect_flags.has_username() {
             remaining_length += self.username.bytes();
         }
-        if self.connect_flags.password() {
+        if self.connect_flags.has_password() {
             remaining_length += self.password.bytes();
         }
 
@@ -410,10 +474,10 @@ impl EncodePacket for ConnectPacket {
 
             self.will_message.encode(v)?;
         }
-        if self.connect_flags.username() {
+        if self.connect_flags.has_username() {
             self.username.encode(v)?;
         }
-        if self.connect_flags.password() {
+        if self.connect_flags.has_password() {
             self.password.encode(v)?;
         }
 
@@ -455,7 +519,7 @@ impl DecodePacket for ConnectPacket {
         }
 
         // If the User Name Flag is set to 0, the Password Flag MUST be set to 0 [MQTT-3.1.2-22].
-        if !connect_flags.username() && connect_flags.password() {
+        if !connect_flags.has_username() && connect_flags.has_password() {
             return Err(DecodeError::InvalidConnectFlags);
         }
 
@@ -479,13 +543,13 @@ impl DecodePacket for ConnectPacket {
             BinaryData::new()
         };
 
-        let username = if connect_flags.username() {
+        let username = if connect_flags.has_username() {
             StringData::decode(ba)?
         } else {
             StringData::new()
         };
 
-        let password = if connect_flags.password() {
+        let password = if connect_flags.has_password() {
             BinaryData::decode(ba)?
         } else {
             BinaryData::new()
@@ -512,15 +576,15 @@ impl DecodePacket for ConnectPacket {
             return Err(DecodeError::InvalidPropertyType);
         }
 
-        Ok(ConnectPacket {
+        Ok(Self {
             protocol_name,
             protocol_level,
-            keep_alive,
             connect_flags,
+            keep_alive,
             properties,
             client_id,
-            will_topic,
             will_properties,
+            will_topic,
             will_message,
             username,
             password,
